--$Revision: 152 $
--$Date: 2009-12-12 21:19:58 +0000 (Sat, 12 Dec 2009) $
--project version: r158
local LSM = LibStub("LibSharedMedia-3.0")

local lastUpdate = 0
function SimpleMD:OnUpdate()
	local doslow
	lastUpdate = lastUpdate + arg1
	if lastUpdate > 0.04 then
		doslow = true
		lastUpdate = 0
	end
	
	local t
	for k, v in pairs(SimpleMD.anchor.timers) do
		if not t then t = GetTime() end
		if (not doslow) and v.slow then
			--do nothing
		elseif v.running then
			v.elapsed = t - v.start
			if v.stop <= t then
				SimpleMD:StopTimerBar(k)
			else
				SimpleMD:UpdateTimerBar(k)
			end
		end
	end	
end

function SimpleMD:ClearTimerBars()
	for i = table.getn(self.anchor.timers), 1, -1 do
		self.anchor.timers[i]:Hide()
		table.insert(self.recyclePile, table.remove(self.anchor.timers))
		table.remove(self.idents, i)
		self.timercount = self.timercount - 1
	end
end

function SimpleMD:ReadinessCheck(name)
	for k, v in pairs(self.idents) do
		if v == name then
			local reverse = self.anchor.timers[k].reversed
			self.anchor.timers[k].timer:SetValue(reverse and 1 or 0)
			self:StopTimerBar(k)
		end
	end
end

function SimpleMD:UpdateTimerBar(index)
	local t = self.anchor.timers[index].time - self.anchor.timers[index].elapsed
	self.anchor.timers[index].slow = t > 11
	
	local newtimetext
	
	local h = floor(t/3600)
	local m = t - (h*3600)
	m = floor(m/60)
	local s = t - ((h*3600) + (m*60))
	
	if h > 0 then
		newtimetext = ("%d:%02d"):format(h, m)
	elseif m > 0 then
		newtimetext = string.format("%d:%02d", m, floor(s))
	elseif s < 10 then
		newtimetext = string.format("%1.1f", s)
	else
		newtimetext = string.format("%.0f", floor(s))
	end
	
	self.anchor.timers[index].timetext:SetText(newtimetext)
	
	local percent = t / self.anchor.timers[index].time
	local reverse = self.anchor.timers[index].reversed
	self.anchor.timers[index].timer:SetValue(reverse and 1-percent or percent)
end

function SimpleMD:StopTimerBar(index)	
	self.anchor.timers[index].running = false
	if not self.anchor.timers[index].stayonscreen then
		for k, v in pairs(self.anchor.timers) do
			if k == 1 then
				--nothing
			elseif k == 2 then
				self.anchor.timers[k]:ClearAllPoints()
				self.anchor.timers[k]:SetPoint("TOP", self.frame, "BOTTOM", 10, 5)
			else
				if self.anchor.timers[index].growup then
					self.anchor.timers[k]:ClearAllPoints()
					self.anchor.timers[k]:SetPoint("BOTTOM", self.anchor.timers[k-1], "TOP", 0, 0)
				else
					self.anchor.timers[k]:ClearAllPoints()
					self.anchor.timers[k]:SetPoint("TOP", self.anchor.timers[k-1], "BOTTOM", 0, 0)
				end
			end
		end
		self:RecycleTimerBar(index)
	else
		self.anchor.timers[index].timetext:SetText("")
		local i = string.find(self.anchor.timers[index].text:GetText(), " ")
		local temp = string.sub(self.anchor.timers[index].text:GetText(), 1, i-1)
		self.anchor.timers[index].text:SetJustifyH("CENTER")
		self.anchor.timers[index].text:SetText(temp.." ready!")
	end
end

function SimpleMD:RecycleTimerBar(index)
	local alias = self.anchor.timers[index]
	if not alias then return end
	
	alias:Hide()
	self.timercount = self.timercount - 1
	table.insert(self.recyclePile, alias)
	table.remove(self.anchor.timers, index)
	table.remove(self.idents, index)
end

function SimpleMD:ReanchorTimerBars(growth)
	for k, v in pairs(self.anchor.timers) do
		if k == 1 then
			if growth then
				self.anchor.timers[k]:ClearAllPoints()
				self.anchor.timers[k]:SetPoint("BOTTOM", self.frame, "TOP", 10, -5)
				self.anchor.timers[k].growup = growth
			else
				self.anchor.timers[k]:ClearAllPoints()
				self.anchor.timers[k]:SetPoint("TOP", self.frame, "BOTTOM", 10, 5)
				self.anchor.timers[k].growup = growth
			end
		elseif k >= 2 then
			if growth then
				self.anchor.timers[k]:ClearAllPoints()
				self.anchor.timers[k]:SetPoint("BOTTOM", self.anchor.timers[k-1], "TOP", 0, 0)
				self.anchor.timers[k].growup = growth
			else
				self.anchor.timers[k]:ClearAllPoints()
				self.anchor.timers[k]:SetPoint("TOP", self.anchor.timers[k-1], "BOTTOM", 0, 0)
				self.anchor.timers[k].growup = growth
			end
		end
	end
end

function SimpleMD:RefreshTimerBar(index, caster, target, runtime, reverse, stay, growth)
	local alias = self.anchor.timers[index]
	if not alias then return end
	
	alias.timer:SetStatusBarColor(self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, 1)
	alias.timer.bg:SetVertexColor(self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	alias.timer.bg:SetAlpha(self.db.profile.bg_alpha)
	alias.timer:SetValue(1)
	alias.text:SetJustifyH("LEFT")
	if target then
		alias.text:SetText(caster.." - "..target)
	else
		alias.text:SetText(caster)
	end
	alias.timetext:SetText(runtime)
	alias.timer:SetStatusBarTexture(LSM:Fetch('statusbar', self.db.profile.texture))
	alias.timer.bg:SetTexture(LSM:Fetch('statusbar', self.db.profile.texture))
	
	alias.elapsed = 0
	alias.time = runtime
	alias.running = false
	alias.reversed = reverse
	alias.stayonscreen = stay
end

function SimpleMD:CreateTimerBar(caster, target, runtime, reverse, stay, growth)
	for k, v in pairs(self.idents) do
		if v == caster then
			self:RefreshTimerBar(k, caster, target, runtime, reverse, stay, growth)
			return
		end
	end
	
	self.timercount = self.timercount + 1
	
	local t
	if #self.recyclePile > 0 then
		t = table.remove(self.recyclePile)
	else
		t = CreateFrame("BUTTON", nil, self.frame)
	end
	
	t:SetWidth(self.db.profile.barWidth * self.db.profile.barScale)
	t:SetHeight(self.db.profile.barHeight * self.db.profile.barScale)
	t:ClearAllPoints()
	if self.timercount == 1 then
		if growth then
			t:SetPoint("BOTTOM", self.frame, "TOP", 10, -5)
		else
			t:SetPoint("TOP", self.frame, "BOTTOM", 10, 5)
		end
	else
		if growth then
			t:SetPoint("BOTTOM", self.anchor.timers[self.timercount-1], "TOP", 0, 0)
		else
			t:SetPoint("TOP", self.anchor.timers[self.timercount-1], "BOTTOM", 0, 0)
		end
	end
	if not t.timer then
		t.timer = CreateFrame("STATUSBAR", nil, t)
	end
	t.timer:ClearAllPoints()
	t.timer:SetPoint("TOPLEFT", t, "TOPLEFT")
	t.timer:SetPoint("BOTTOMRIGHT", t, "BOTTOMRIGHT")
	t.timer:SetStatusBarTexture(LSM:Fetch('statusbar', self.db.profile.texture))
	t.timer:SetStatusBarColor(self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, 1)
	t.timer:SetMinMaxValues(0, 1)
	t.timer:SetValue(1)
	
	if not t.timer.bg then
		t.timer.bg = t.timer:CreateTexture(nil, "BACKGROUND")
	end
	t.timer.bg:ClearAllPoints()
	t.timer.bg:SetAllPoints(t.timer)
	t.timer.bg:SetTexture(LSM:Fetch('statusbar', self.db.profile.texture))
	t.timer.bg:SetVertexColor(self.db.profile.BGr, self.db.profile.BGg, self.db.profile.BGb)
	t.timer.bg:SetAlpha(self.db.profile.bg_alpha)
	
	if not t.icon then
		t.icon = t:CreateTexture()
	end
	t.icon:SetTexture(self.tex)
	t.icon:ClearAllPoints()
	t.icon:SetPoint("RIGHT", t, "LEFT", 0, 0)
	t.icon:SetTexCoord(.07, .93, .07, .93)
	t.icon:SetWidth(self.db.profile.barHeight * self.db.profile.barScale)
	t.icon:SetHeight(self.db.profile.barHeight * self.db.profile.barScale)
	
	if not t.text then
		t.text = t.timer:CreateFontString(nil, "LOW")
	end
	t.text:SetWidth(self.db.profile.barWidth * self.db.profile.barScale)
	t.text:SetHeight(self.db.profile.barHeight * self.db.profile.barScale)
	t.text:SetFont(LSM:Fetch('font', self.db.profile.font), self.db.profile.textSize)
	t.text:SetTextColor(self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB, 1)
	t.text:SetJustifyH("LEFT")
	t.text:SetJustifyV("MIDDLE")
	t.text:ClearAllPoints()
	t.text:SetPoint("TOP", t.timer, "TOP", 5, 0)
	if target then
		t.text:SetText(caster.."  -  "..target)
	else
		t.text:SetText(caster)
	end
	
	if not t.timetext then
		t.timetext = t.timer:CreateFontString(nil, "LOW")
	end
	t.timetext:SetWidth(self.db.profile.barWidth * self.db.profile.barScale)
	t.timetext:SetHeight(self.db.profile.barHeight * self.db.profile.barScale)
	t.timetext:SetFont(LSM:Fetch('font', self.db.profile.font), self.db.profile.textSize)
	t.timetext:SetTextColor(self.db.profile.txtR, self.db.profile.txtG, self.db.profile.txtB, 1)
	t.timetext:SetJustifyH("RIGHT")
	t.timetext:SetJustifyV("MIDDLE")
	t.timetext:ClearAllPoints()
	t.timetext:SetPoint("TOP", t.timer, "TOP", -5, 0)
	t.timetext:SetText(runtime)
	t.timetext:Show()
	
	local curTime = GetTime()
	t.start = curTime
	t.stop = curTime + runtime
	t.elapsed = 0
	t.time = runtime
	t.running = true
	t.reversed = reverse
	t.growup = growth
	t.stayonscreen = stay
	
	t:SetFrameStrata("LOW")
	t:Show()
	
	local function DisableMouse(Frame)
		if Frame:IsMouseEnabled() then
			Frame:EnableMouse(false)
		end
		if Frame:GetChildren() then
			for _,child in pairs({Frame:GetChildren()}) do
				if child:IsMouseEnabled() then
					child:EnableMouse(false)
				end
				DisableMouse(child)
			end 
		end
	end
	DisableMouse(t)
	
	table.insert(self.anchor.timers, t)
	table.insert(self.idents, caster)
end

function SimpleMD:CreateAnchor(text)
	local f = CreateFrame("BUTTON", "smdAnchor", UIParent)
	local g = CreateFrame("FRAME", nil, UIParent)
	f:SetWidth(200)
	f:SetHeight(25)
	g:SetWidth(200)
	g:SetHeight(25)
	
	if self.db.profile.position.x and self.db.profile.position.y and self.db.profile.position.anchor and self.db.profile.position.point then
		f:ClearAllPoints()
		g:ClearAllPoints()
		f:SetPoint(self.db.profile.position.point, UIParent, self.db.profile.position.anchor, self.db.profile.position.x, self.db.profile.position.y)
		g:SetPoint(self.db.profile.position.point, UIParent, self.db.profile.position.anchor, self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER")
		g:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	f:SetScript("OnDragStart", function() f:StartMoving() end )
	f:SetScript("OnDragStop",
		function ()
			f:StopMovingOrSizing()
			local point, _, anchor, x, y = f:GetPoint()
			SimpleMD.db.profile.position.x = math.floor(x)
			SimpleMD.db.profile.position.y = math.floor(y)
			SimpleMD.db.profile.position.anchor = anchor
			SimpleMD.db.profile.position.point = point
			g:ClearAllPoints()
			g:SetPoint(point, UIParent, anchor, x, y)
		end)
	
	f:SetBackdrop({
		bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
		
	f:SetBackdropColor(0, 1, 0, .65)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetClampedToScreen(true)
	f:EnableMouse(true)
	
	f.text = f:CreateFontString(nil, "OVERLAY")
	f.text:SetFontObject(GameFontNormalSmall)
	f.text:ClearAllPoints()
	f.text:SetTextColor(1, 1, 1, 1)
	f.text:SetWidth(200)
	f.text:SetHeight(25)
	f.text:SetPoint("TOP", f, "TOP")
	f.text:SetJustifyH("CENTER")
	f.text:SetJustifyV("MIDDLE")
	f.text:SetText(text)
	
	f:Hide()
	g:Show()
	
	g:SetScript("OnUpdate", self.OnUpdate)
	g:EnableMouse(false)
	
	return f, g
end
